/*********************************************************************
*                SEGGER Microcontroller GmbH & Co. KG                *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2017 SEGGER Microcontroller GmbH & Co. KG         *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 4.34                                             *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : BSP.c
Purpose : BSP for PIC32MX360F512L and PIC32 Explorer 16 eval board
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "BSP.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/

#define TRISA         (*(volatile unsigned int *) (0xBF886000u)) // Port A data direction register 
#define PORTA         (*(volatile unsigned int *) (0xBF886010u)) // Port A data register           
#define PORTACLR      (*(volatile unsigned int *) (0xBF886014u)) // Port A data clear register           
#define PORTASET      (*(volatile unsigned int *) (0xBF886018u)) // Port A data set register           
#define PORTAINV      (*(volatile unsigned int *) (0xBF88601Cu)) // Port A data invert register           

#define LED0_BIT      (2u)                                       // LED2  
#define LED1_BIT      (3u)                                       // LED3  
#define LED2_BIT      (4u)                                       // LED4  
#define LED3_BIT      (5u)                                       // LED5  
#define LEDMASK_ALL   ((1u << LED0_BIT) | (1u << LED1_BIT) | (1u << LED2_BIT) | (1u << LED3_BIT))

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_Init()
*/
void BSP_Init(void) {
  TRISA &= ~LEDMASK_ALL;  // Enable LED output ports
  PORTA &= ~LEDMASK_ALL;  // Switch off all LEDs
}

/*********************************************************************
*
*       BSP_SetLED()
*/
void BSP_SetLED(int Index) {
  if (Index < 4) {
    PORTACLR = (1u << (Index + LED0_BIT));
  }
}

/*********************************************************************
*
*       BSP_ClrLED()
*/
void BSP_ClrLED(int Index) {
  if (Index < 4) {
    PORTASET = (1u << (Index + LED0_BIT));
  }
}

/*********************************************************************
*
*       BSP_ToggleLED()
*/
void BSP_ToggleLED(int Index) {
  if (Index < 4) {
    PORTAINV = (1u << (Index + LED0_BIT));
  }
}

/****** End Of File *************************************************/
