# ################################################################### 
# #                                                                 #   
# #               SEGGER MICROCONTROLLER GmbH & Co KG               #
# #       Solutions for real time microcontroller applications      #
# ###################################################################
# #                                                                 #
# #     (C) 1996-2010   SEGGER Microcontroller Systeme GmbH         #
# #                                                                 #
# #     www.segger.com  Support: support@segger.com                 #
# #                                                                 #
# ###################################################################
#
# --------------------------------------------------------------------
# File        : OS_ISR.S
# Purpose     : Assembler interrupt handler for PIC32 and XC32 compiler
# ------------- END-OF-HEADER ----------------------------------------

# ####################################################################
# #
# #     Const defines
# #
# ####################################################################

#define OS_INT_PRIORITY_1  0x0400
#define OS_INT_PRIORITY_2  0x0800
#define OS_INT_PRIORITY_3  0x0C00
#define OS_INT_PRIORITY_4  0x1000
#define OS_INT_PRIORITY_5  0x1400
#define OS_INT_PRIORITY_6  0x1800
#define OS_INT_PRIORITY_7  0x1C00


#define _C0_SR              $12       // Coprocessor 0, status register 
#define _C0_EPC             $14       // C0 exception (saved) link pointer

# ####################################################################
# #
# #     Add postfix for public
# #
# ####################################################################

# Not currently required, this section is for compatibilty only

# ####################################################################
# #
# #     Forward declarations of segments used
# #
# ####################################################################

# ####################################################################
# #
# #     Publics
# #
# ####################################################################
         .global  OS_SysTick_ISR
         .global  OS_Uart_ISR

# ####################################################################
# #
# #     Externals, code
# #
# ####################################################################
         .extern  OS_SysTick
         .extern  OS_Uart

# ####################################################################
# #
# #     Externals, data
# #
# ####################################################################

# ####################################################################
# #
# #     MACROS to simplify assembler code
# #
# ####################################################################
         .macro OS_CALL_ISR funcptr, prio
         mfc0      $k0,_C0_SR                  
         mfc0      $k1,_C0_EPC        
# Make new stack frame
         addiu     $sp,$sp,-92       
# Save EPC and Status register on stack
         sw        $k0,84($sp)       
         sw        $k1,88($sp)       
# Set new status register value, interrupts are still disabled
         addiu     $k1, $0, 0xFFFFC3FF  
         and       $k0,$k1,$k0
         ori       $k0,$k0,\prio
         mtc0      $k0,_C0_SR         
# Save all scratch registers         
         sw        $at,0($sp)        
         sw        $v0,4($sp)        
         sw        $v1,8($sp)        
         sw        $a0,12($sp)        
         sw        $a1,16($sp)        
         sw        $a2,20($sp)        
         sw        $a3,24($sp)        
         sw        $t0,28($sp)        
         sw        $t1,32($sp)        
         sw        $t2,36($sp)        
         sw        $t3,40($sp)        
         sw        $t4,44($sp)        
         sw        $t5,48($sp)        
         sw        $t6,52($sp)        
         sw        $t7,56($sp)        
         sw        $t8,60($sp)        
         sw        $t9,64($sp)        
         sw        $fp,68($sp)        
         sw        $ra,72($sp)        
         mflo      $v0                
         mfhi      $v1                
         sw        $v0,76($sp)       
         sw        $v1,80($sp)

# If (RegionCnt == 0) {
         jal       OS_GetRegionCnt
         nop
         bne       $v0, $0, __\funcptr
# OS_ISR_StackFrameAddr = sp + 84}
         addiu     $k0, $sp, 84
         sw        $k0, OS_ISR_StackFrameAddr
__\funcptr:
# Call the C function to handle the interrupt
         jal       \funcptr
         nop    
# Disable interrupt
         di        
# Restore all scratch register
         lw        $at,0($sp)        
         lw        $a0,12($sp)        
         lw        $a1,16($sp)        
         lw        $a2,20($sp)        
         lw        $a3,24($sp)        
         lw        $t0,28($sp)        
         lw        $t1,32($sp)        
         lw        $t2,36($sp)        
         lw        $t3,40($sp)        
         lw        $t4,44($sp)        
         lw        $t5,48($sp)        
         lw        $t6,52($sp)        
         lw        $t7,56($sp)        
         lw        $t8,60($sp)        
         lw        $t9,64($sp)        
         lw        $fp,68($sp)        
         lw        $ra,72($sp)        
         lw        $v0,76($sp)       
         lw        $v1,80($sp)
         mtlo      $v0                
         mthi      $v1                
         lw        $v0,4($sp)        
         lw        $v1,8($sp)        
# Get saved EPC and status register from stack
         lw        $k0,84($sp)
         lw        $k1,88($sp)
# Correct stack pointer
         addiu     $sp,$sp,92
# Restore EPC and status register
         mtc0      $k0,_C0_SR
         ehb
         mtc0      $k1,_C0_EPC
         ehb
         eret  
         .endm

# ####################################################################
# #
# #     CODE segment
# #
# ####################################################################
          .text
          .align 2

# ####################################################################
# #
# #       void OS_SysTick_ISR(void)
# #
# #       Wrapper function for OS_Systick.
# #       This function saves and restores all necessary registers.
# #       Interrupts are not enabled (MPLAB compiler does this in 
# #       interrupt function prolog). 
# #
OS_SysTick_ISR:
         OS_CALL_ISR  OS_Systick OS_INT_PRIORITY_1

# ####################################################################
# #
# #       void OS_UartHandler_ISR(void)
# #
# #       Wrapper function for OS_UartHandler.
# #       This function saves and restores all necessary registers.
# #       Interrupts are not enabled (MPLAB compiler does this in 
# #       interrupt function prolog). 
# #
OS_Uart_ISR:
         OS_CALL_ISR  OS_Uart OS_INT_PRIORITY_1

# ####  End of file  #################################################

