/*********************************************************************
*                SEGGER Microcontroller GmbH & Co. KG                *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2017 SEGGER Microcontroller GmbH & Co. KG         *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 4.34                                             *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : RTOSInit_PIC32MX795F512L.c for PIC32MX795F512L CPU
Purpose : Initializes and handles the hardware for embOS as far
          as required by embOS
          Feel free to modify this file acc. to your target system.
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "RTOS.h"

/*********************************************************************
*
*       Configuration
*
**********************************************************************
*/

/*********************************************************************
*
*       PLL Settings
*
* Configuration Bit settings
*  System Clock = 80 MHz,  Peripherial Bus = 40 MHz
*  Primary Osc w/PLL (XT+,HS+,EC+PLL)
*  Input Divider    2x Divider
*  Multiplier      20x Multiplier
*  WDT disabled
*  Other options are don't care
*/
#pragma config FPLLODIV = DIV_1
#pragma config FPLLMUL  = MUL_20
#pragma config FPLLIDIV = DIV_2
#pragma config FNOSC    = PRIPLL
#pragma config POSCMOD  = HS
#pragma config FWDTEN   = OFF
#if DEBUG
  #pragma config DEBUG    = ON
#endif

/*********************************************************************
*
*       Clock frequency settings (configuration)
*/
#ifndef   OS_FSYS                      /* CPU Main clock frequency   */
  #define OS_FSYS 80000000uL           /* Assume 80MHz               */
#endif

#ifndef OS_PCLK_TIMER                  /* Peripheral clock for timer */
  #define OS_PCLK_TIMER (OS_FSYS / 2)  /* Assume CP0 count is used   */
#endif

#ifndef OS_PCLK_UART                   /* Peripheral clock for UART  */
  #define OS_PCLK_UART  (OS_FSYS)      /* Assume CLK18 used          */
#endif

#ifndef OS_TICK_FREQ
  #define OS_TICK_FREQ  (1000u)
#endif

#define OS_TIMER_RELOAD (OS_PCLK_TIMER / OS_TICK_FREQ)

/*********************************************************************
*
*       Configuration of communication to embOSView
*/
#ifndef   OS_VIEW_IFSELECT
  #define OS_VIEW_IFSELECT  OS_VIEW_IF_UART
#endif

/*********************************************************************
*
*       UART settings for embOSView
*       If you do not want (or can not due to hardware limitations)
*       to dedicate a UART to embOSView, please define it to be -1
*       Currently UART3 is supported and the standard
*       setup enables UART3 per default
*/
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
  #ifndef OS_UART
    #define OS_UART (3)
  #endif

  #ifndef OS_BAUDRATE
    #define OS_BAUDRATE (38400)
  #endif
#endif

/****** End of configurable options *********************************/

/*********************************************************************
*
*       Local defines (sfrs and addresses used in RTOSInit.c)
*
**********************************************************************
*/
/* System and interrupt defines */
#define IEC0SET                 (*(volatile OS_U32* )0xBF881068u)
#define IEC1CLR                 (*(volatile OS_U32* )0xBF881074u)
#define IEC1SET                 (*(volatile OS_U32* )0xBF881078u)
#define IPC0SET                 (*(volatile OS_U32* )0xBF881098u)
#define IFS0CLR                 (*(volatile OS_U32* )0xBF881034u)
#define IFS1                    (*(volatile OS_U32* )0xBF881040u)
#define IFS1CLR                 (*(volatile OS_U32* )0xBF881044u)
#define INTCONSET               (*(volatile OS_U32* )0xBF881008u)
#define CTIE                    0u
#define SYSTICK_PRIORITY        1u
#define SYSTICK_SUBPRIORITY     0u
#define SYSKEY                  (*(volatile OS_U32* )0xBF80F230u)
#define OSCCONCLR               (*(volatile OS_U32* )0xBF80F004u)
#define OSCCONSET               (*(volatile OS_U32* )0xBF80F008u)

/* Uart defines */
#define U3AMODE                 (*(volatile OS_U32* )0xBF806800u)
#define U3ASTA                  (*(volatile OS_U32* )0xBF806810u)
#define U3ASTACLR               (*(volatile OS_U32* )0xBF806814u)
#define U3ATXREG                (*(volatile OS_U32* )0xBF806820u)
#define U3ARXREG                (*(volatile OS_U32* )0xBF806830u)
#define U3ABRGSET               (*(volatile OS_U32* )0xBF806848u)
#define IPC7SET                 (*(volatile OS_U32* )0xBF881108u)
#define UART3A_PRIORITY         1u
#define UART3A_SUBPRIORITY      0u
#define U3AEIF                  (1u << 8u)
#define U3ARXIF                 (1u << 9u)
#define U3ATXIF                 (1u << 10u)

#define FLASH_SPEED_HZ          30000000u //Max Flash speed
#define PB_BUS_MAX_FREQ_HZ      80000000u //Max Peripheral bus speed
#define OSC_PB_DIV_1            (0u << _OSCCON_PBDIV_POSITION)
#define OSC_PB_DIV_2            (1u << _OSCCON_PBDIV_POSITION)
#define BMXCONCLR               (*(volatile unsigned int* )0xBF882004u)
#define CHECON                  (*(volatile unsigned int* )0xBF884000u)
#define CHE_CONF_PF_ALL         (3u << _CHECON_PREFEN_POSITION)
#define mSYSTEMUnlock(intStat, dmaSusp) do { intStat = INTDisableInterrupts(); \
                                             dmaSusp = DmaSuspend();           \
                                             SYSKEY = 0u,                      \
                                             SYSKEY = 0xAA996655u,             \
                                             SYSKEY = 0x556699AAu;             \
                                           } while(0)
#define mSYSTEMLock(intStat, dmaSusp) do {   SYSKEY = 0x33333333u;             \
                                             DmaResume(dmaSusp);               \
                                             INTRestoreInterrupts(intStat);    \
                                           } while (0)

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

/*********************************************************************
*
*       _InitVIC()
*
*  Function description
*    Set the CP0 registers for multi-vector interrupt Place
*    Vector Spacing to 32 bytes
*/
static void __attribute__((nomips16)) _InitVIC(void) {
  unsigned int temp;
  OS_MIPS_SetIntCtl(0x00000020u); // Set the Vector Spacing to non-zero value
  temp  = OS_MIPS_GetCause();
  temp |= 0x00800000u;            // Set IV
  OS_MIPS_SetCause(temp);         // Update Cause
  temp  = OS_MIPS_GetStatus();
  temp &= 0xFFBFFFFDu;            // Clear BEV and EXL
  OS_MIPS_SetStatus(temp);
  INTCONSET = 0x1000u;            // Set MVEC bit
}

/*********************************************************************
*
*       Legacy plib library functions
*/
static int DmaSuspend(void) {
  int suspSt;
  if( !(suspSt = DMACONbits.SUSPEND)) {
    DMACONSET=_DMACON_SUSPEND_MASK;     // suspend
    while(!(DMACONbits.SUSPEND));       // wait to be actually suspended
  }
  return suspSt;
}

static void DmaResume(int susp) {
  if(susp) {
    DmaSuspend();
  } else {
    DMACONCLR = _DMACON_SUSPEND_MASK;   // resume DMA activity
  }
}

static void OSCSetPBDIV(unsigned int oscPbDiv) {
  unsigned int dma_status;
  unsigned int int_status;
  __OSCCONbits_t oscBits;

  mSYSTEMUnlock(int_status, dma_status);

  oscBits.w      = OSCCON;    // read to be in sync. flush any pending write
  oscBits.PBDIV  = 0;
  oscBits.w     |= oscPbDiv;
  OSCCON         = oscBits.w; // write back
  oscBits.w      = OSCCON;    // make sure the write occurred before returning from this function

  mSYSTEMLock(int_status, dma_status);
}

static unsigned int SYSTEMConfigPB(unsigned int sys_clock) {
  unsigned int pb_div;
  unsigned int pb_clock;

  pb_clock = sys_clock;

  if (sys_clock > PB_BUS_MAX_FREQ_HZ) {
    pb_div=OSC_PB_DIV_2;
    pb_clock >>= 1;
  } else {
    pb_div=OSC_PB_DIV_1;
  }

  OSCSetPBDIV(pb_div);

  return pb_clock;
}

static unsigned int SYSTEMConfigWaitStatesAndPB(unsigned int sys_clock) {
#ifdef _PCACHE
  unsigned int wait_states;
#endif
  unsigned int pb_clock;
  unsigned int int_status;

  pb_clock = SYSTEMConfigPB(sys_clock);

  // set the flash wait states based on 1 wait state
  // for every 20 MHz
#ifdef _PCACHE
  wait_states = 0;

  while(sys_clock > FLASH_SPEED_HZ) {
    wait_states++;
    sys_clock -= FLASH_SPEED_HZ;
  }

  int_status=INTDisableInterrupts();
  CHECON = wait_states;
  INTRestoreInterrupts(int_status);
#endif

  return pb_clock;
}

static unsigned int SYSTEMConfigPerformance(unsigned int sys_clock) {
  // set up the wait states
  unsigned int pb_clk;
#ifdef _PCACHE
  unsigned int cache_status;
#endif
  unsigned int int_status;

  pb_clk     = SYSTEMConfigWaitStatesAndPB(sys_clock);
  int_status = INTDisableInterrupts();
  BMXCONCLR  = _BMXCON_BMXWSDRM_MASK;

#ifdef _PCACHE
  cache_status = CHECON;
  cache_status |= CHE_CONF_PF_ALL;
  CHECON = cache_status;
  CheKseg0CacheOn();
#endif

  INTRestoreInterrupts(int_status);

  return pb_clk;
}

/*********************************************************************
*
*       _OS_GetHWTimerCycles()
*
*  Function description
*    Returns the current hardware timer count value
*
*  Return value
*    Current timer count value
*/
static unsigned int _OS_GetHWTimerCycles(void) {
  OS_I32 Cnt;
  Cnt  = (OS_MIPS_GetCompare() - OS_MIPS_GetCount());
  if (Cnt < 0) {  // Missed a counter interrupt, adjust time
    Cnt = 0 - Cnt;
  } else {
    Cnt = OS_TIMER_RELOAD - Cnt;
  }
  return (unsigned int)Cnt;
}

/*********************************************************************
*
*       _OS_GetHWTimer_IntPending()
*
*  Function description
*    Returns if the hardware timer interrupt pending flag is set
*
*  Return value
*    == 0; Interrupt pending flag not set
*    != 0: Interrupt pending flag set
*/
static unsigned int _OS_GetHWTimer_IntPending(void) {
  OS_I32 Cnt;
  Cnt  = OS_MIPS_GetCompare() - OS_MIPS_GetCount();
  if (Cnt < 0) {
    return 1;
  } else {
    return 0;
  }
}

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/
void __attribute__( (interrupt(IPL1SOFT), vector(0))) OS_SysTick_ISR( void );

/*********************************************************************
*
*       OS_Systick()
*
*  Function description
*    This is the code that gets called when the processor receives a
*    _SysTick exception. SysTick is used as OS timer tick.
*/
void OS_Systick(void) {
  OS_I32 t, t1;
  IFS0CLR = 0x00000001;      // reset core timer interrupt pending flag
  OS_EnterNestableInterrupt();

  t  = OS_MIPS_GetCompare();
  do {
    t +=  OS_TIMER_RELOAD;
    OS_TICK_Handle();
    t1 = OS_MIPS_GetCount() + 1000;
  } while ((t - t1) < 0);
  OS_MIPS_SetCompare(t);

  OS_LeaveNestableInterrupt();
}

/*********************************************************************
*
*       OS_InitHW()
*
*  Function description
*    Initialize the hardware (timer) required for embOS to run.
*    May be modified, if an other timer should be used
*/
void OS_InitHW(void) {
  //
  // Structure with information about timer frequency, tick frequency, etc.
  // for micro second precise system time.
  //
  OS_SYSTIMER_CONFIG SysTimerConfig = {OS_PCLK_TIMER, OS_TICK_FREQ, 1, _OS_GetHWTimerCycles, _OS_GetHWTimer_IntPending};

  OS_IncDI();
  //
  // Initialize timer for embOS, assuming PLL is already initialized
  //
  SYSKEY    = 0;                                                   // Unlock register
  SYSKEY    = 0xAA996655;
  SYSKEY    = 0x556699AA;
  OSCCONCLR = (0x03 << 19);                                        // Configures OSC, sets CPU osc source
  OSCCONSET = ((OSC_PB_DIV_2)<<19);
  SYSKEY    = 0x33333333;                                          // Lock register
  SYSTEMConfigPerformance(80000000L);                              // Setup cache, wait states
  CHECON = 2;                                                      // Set 2 wait states for flash
  OS_MIPS_SetCompare(OS_MIPS_GetCount() + OS_TIMER_RELOAD);        // Set compare register
  _InitVIC();                                                      // Setup vectored interrupt controller
  IPC0SET |= (SYSTICK_PRIORITY << 2u) | SYSTICK_SUBPRIORITY;       // Set priority
  IEC0SET |= (1u << CTIE);                                         // Enable core time interrupt in interrupt vector controller
  //
  // Setup values for usec precise system time functions
  //
  OS_Config_SysTimer(&SysTimerConfig);
  //
  // Initialize the optional communication for embOSView
  //
#if (OS_VIEW_IFSELECT != OS_VIEW_DISABLED)
  OS_COM_Init();
#endif
  OS_DecRI();
}

/*********************************************************************
*
*       OS_Idle()
*
*  Function description
*    This is basically the "core" of the idle loop.
*    This core loop can be changed, but:
*    The idle loop does not have a stack of its own, therefore no
*    functionality should be implemented that relies on the stack
*    to be preserved. However, a simple program loop can be programmed
*    (like toggling an output or incrementing a counter)
*/
void OS_Idle(void) {    // Idle loop: No task is ready to execute
  while (1) {           // Nothing to do ... wait for interrupt
    #if (OS_DEBUG == 0)
                        // Switch CPU into sleep mode
    #endif
  }
}

/*********************************************************************
*
*       OS_GetTime_Cycles()
*
*  Function description
*    This routine is required for task-info via embOSView or high
*    resolution time measurement functions.
*    It returns the system time in timer clock cycles.
*/
OS_U32 OS_GetTime_Cycles(void) {
  OS_U32 Time;
  OS_I32 Cnt;

  Time = OS_GetTime32();
  Cnt  = (OS_MIPS_GetCompare() - OS_MIPS_GetCount());
  if (Cnt < 0) {  // Missed a counter interrupt, adjust time
    Time++;
    Cnt = 0 - Cnt;
  } else {
    Cnt = OS_TIMER_RELOAD - Cnt;
  }
  return (OS_TIMER_RELOAD * Time) + Cnt;
}

/*********************************************************************
*
*       OS_ConvertCycles2us()
*
*  Function description
*    Convert Cycles into micro seconds.
*
*    If your clock frequency is not a multiple of 1 MHz,
*    you may have to modify this routine in order to get proper
*    diagnostics.
*
*    This routine is required for profiling or high resolution time
*    measurement only. It does not affect operation of the OS.
*/
OS_U32 OS_ConvertCycles2us(OS_U32 Cycles) {
  return (Cycles / (OS_TIMER_RELOAD / 1000u));
}

/*********************************************************************
*
*       Optional communication with embOSView
*
**********************************************************************
*/
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)

#define OS_BAUDDIVIDE ((OS_PCLK_UART / (OS_BAUDRATE * 16u)) - 1u)

void __attribute__( (interrupt(IPL1SOFT), vector(32))) OS_Uart_ISR( void );

/*********************************************************************
*
*       OS_UartHandler()
*       Uart 3 Handler function for Tx, Rx and Error interrupt
*/
void OS_Uart(void) {
  OS_U32 c;
  OS_EnterInterrupt();
  if (IFS1 & U3AEIF) {                                   // Rx error interrupt?
    if (IFS1 & U3ARXIF) {                                // Rx interrupt?
      c = U3ARXREG;                                      // Discard received byte
      IFS1CLR = U3ARXIF;                                 // Clear uart error pending flag
    }
    U3ASTACLR = ((1 << 1) | (1 << 2) | (1 << 3));        // Clear all uart error status bits
    IFS1CLR = U3AEIF;                                    // Clear uart error pending flag
  } else {
    if (IFS1 & U3ARXIF) {                                // Any byte received?
      c = U3ARXREG;                                      // Read received byte
      OS_OnRx(c);
      IFS1CLR = U3ARXIF;                                 // Clear uart rx pending flag
    }
  }

  if (IFS1 & U3ATXIF) {                                  // Tx interrupt?
    IFS1CLR = U3ATXIF;                                   // Clear uart tx pending flag
    if (OS_OnTx()) {
      IEC1CLR = U3ATXIF;                                 // Disable tx interrupt
    }
  }
  OS_LeaveInterrupt();
}

/*********************************************************************
*
*       OS_COM_Send1()
*
*  Function description
*       Send one character via UART.
*       Never call this function from your application
*/
void OS_COM_Send1(OS_U8 c) {
  IEC1SET = (1 << 10);                                      // Enable tx interrupt
  U3ATXREG = c;
}

/*********************************************************************
*
*       OS_COM_Init()
*
*  Function description
*    Initializes communication for embOSView
*/
void OS_COM_Init(void) {
  U3AMODE     = 0x8000;                                              // Initialize UART 8-N-1
  U3ABRGSET   = OS_BAUDDIVIDE;                                       // Set baud rate
  U3ASTA      = (1 << 15) | (1 << 12) | (1 << 10);                   // Enable Rx and Tx
  IPC8SET     = (UART3A_PRIORITY << 2) | UART3A_SUBPRIORITY;         // Set priority
  IEC1SET     = (1 << 8) | (1 << 9);                                 // Enable Rx and Error interrupts
}

#elif (OS_VIEW_IFSELECT == OS_VIEW_DISABLED)

/*********************************************************************
*
*       OS_COM_Send1()
*
*  Function description
*    Dummy routine.
*/
void OS_COM_Send1(OS_U8 c) {
  OS_USEPARA(c);           // Avoid compiler warning
  OS_COM_ClearTxActive();  // Let embOS know that Tx is not busy
}

/*********************************************************************
*
*       OS_Uart()
*
*  Function description
*    Dummy routine.
*/
void OS_Uart(void) {
}
  
#else
  #error "Selected embOSView interface is currently not supported."
#endif

/****** End Of File *************************************************/
