/*********************************************************************
*                SEGGER Microcontroller GmbH & Co. KG                *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2017 SEGGER Microcontroller GmbH & Co. KG         *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 4.34                                             *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : RTOSInit_PIC32MZ2048EC.c for PIC32MZ2048EC CPU
Purpose : Initializes and handles the hardware for embOS as far
          as required by embOS
          Feel free to modify this file acc. to your target system.
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "RTOS.h"

/*********************************************************************
*
*       Configuration
*
**********************************************************************
*/

//
// Pragmas for configuration bits
// Auto-detected by MPLAB X and merged into compiled code
//
//
// DEVCFG3
//
#pragma config FUSBIDIO = OFF            // USB USBID Selection (Controlled by Port Function)
#pragma config IOL1WAY  = OFF            // Peripheral Pin Select Configuration (Allow only one reconfiguration)
#pragma config PMDL1WAY = OFF            // Peripheral Module Disable Configuration (Allow only one reconfiguration)
#pragma config PGL1WAY  = OFF            // Permission Group Lock One Way Configuration (Allow only one reconfiguration)
#pragma config FETHIO   = OFF            // Ethernet I/O Pin Select (Default Ethernet I/O)
#pragma config FMIIEN   = OFF            // Ethernet RMII/MII Enable (MII Enabled)
#pragma config USERID   = 0xFFFF         // 16-bit user ID
//
// DEVCFG2
// Clock tree / PLL:
//   Clock => / FPLLIDIV => FIN => * FPPLMUL => FVCO => PLLODIV => FSYS (SPLL)
//   FIN     PLL Input Frequency Range  5-64 MHz   ECPLL, HSPLL, FRCPLL modes
//   FVCO    PLL VCO Frequency Range    350-700 MHz
//   FPLL    PLL Output Frequency Range 10-200 MHz
//   FSYS System Frequency              DC-200 MHz
//   FREF Reference Clock Frequency     0-50 MHz
//
#pragma config UPLLEN   = OFF            // USB PLL Enable
#pragma config UPLLFSEL = FREQ_24MHZ     // USB PLL Input Frequency Selection
#pragma config FPLLODIV = DIV_2          // System PLL Output Divisor bits
#pragma config FPLLMULT = MUL_50          // System PLL Multiplier
#pragma config FPLLICLK = PLL_FRC //PLL_POSC       // System PLL Input Clock Selection
#pragma config FPLLRNG  = RANGE_8_16_MHZ   // System PLL Input Clock Frequency Range
#pragma config FPLLIDIV = DIV_1          // PLL Input Divider bits
//
// DEVCFG1
//
#pragma config FDMTEN   = OFF            // Deadman Timer Enable bit
#pragma config DMTCNT   = DMT31          // Deadman Timer Count Select bits
#pragma config FWDTWINSZ = WINSZ_75      // Watchdog Timer Window Size in %
#pragma config FWDTEN   = OFF            // Watchdog Timer Enable
#pragma config WINDIS   = NORMAL         // Watchdog Timer Window Mode (NORMAL == non-Window mode)
#pragma config WDTSPGM  = STOP           // Watchdog Timer Stop During Flash Programming
#pragma config WDTPS    = PS1048576      // Watchdog Timer Postscaler (1:xx)
#pragma config FCKSM    = CSDCMD         // Clock Switching and Monitor Selection
#pragma config OSCIOFNC = OFF            // CLKO Output Signal Active on the OSCO Pin
#pragma config POSCMOD  = EC             // Primary Oscillator Configuration (EC == External clock input, HS = High-speed oscillator)
#pragma config IESO     = OFF            // Internal/External Switch Over
#pragma config FSOSCEN  = OFF            // Secondary Oscillator Enable
#pragma config DMTINTV  = WIN_127_128    // DMT Count Window Interval
#pragma config FNOSC    = SPLL           // Oscillator Selection Bits => SYSCLK source selection
//
// DEVCFG0
//
#pragma config EJTAGBEN = NORMAL        // Boot device with normal or reduced EJTAG functionality
#pragma config DBGPER   = PG_ALL        // Debug Mode CPU Access Permission bits
#pragma config FSLEEP   = OFF           // Flash sleep Mode bit (OFF == Power down flash in sleep mode)
#pragma config FECCCON  = OFF_UNLOCKED  // Dynamic Flash ECC Configuration bits
#ifdef MICROMIPS
  #pragma config BOOTISA  = MICROMIPS   // Boot ISA Selection bit (Boot in MIPS32 or microMIPS mode)
#else
  #pragma config BOOTISA  = MIPS32      // Boot ISA Selection bit (Boot in MIPS32 or microMIPS mode)
#endif
#pragma config TRCEN    = OFF           // Trace enable bit
#pragma config ICESEL   = ICS_PGx2      // In-circuit debugger communication channel select bits
#pragma config JTAGEN   = ON            // 4-wire JTAG enable bit
#pragma config DEBUG    = OFF           // Background debugger enable bits
//
// DEVCP0
//
#pragma config CP = OFF                 // Code Protect (Protection Disabled)

/*********************************************************************
*
*       Clock frequency settings (configuration)
*/
#ifndef   OS_FSYS                      /* CPU Main clock frequency   */
  #define OS_FSYS 200000000uL          /* Assume 200MHz              */
#endif

#ifndef OS_PCLK_TIMER                  /* Peripheral clock for timer */
  #define OS_PCLK_TIMER (OS_FSYS / 2)  /* Assume CP0 count is used   */
#endif

#ifndef OS_PCLK_UART                   /* Peripheral clock for UART  */
  #define OS_PCLK_UART  (OS_FSYS)      /* Assume CLK18 used          */
#endif

#ifndef OS_TICK_FREQ
  #define OS_TICK_FREQ  (1000u)
#endif

#define OS_TIMER_RELOAD (OS_PCLK_TIMER / OS_TICK_FREQ)

/*********************************************************************
*
*       Configuration of communication to embOSView
*/
#ifndef   OS_VIEW_IFSELECT
  #define OS_VIEW_IFSELECT  OS_VIEW_IF_DISABLED
#endif

/*********************************************************************
*
*       UART settings for embOSView
*       If you do not want (or can not due to hardware limitations)
*       to dedicate a UART to embOSView, please define it to be -1
*       Currently no uart is supported
*/
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
  #ifndef OS_UART
    #define OS_UART (-1)   // No Uart support so far
  #endif

  #ifndef OS_BAUDRATE
    #define OS_BAUDRATE (38400)
  #endif
#endif

/****** End of configurable options *********************************/

/*********************************************************************
*
*       Local defines (sfrs and addresses used in RTOSInit.c)
*
**********************************************************************
*/
/* System and interrupt defines */
#define IEC0                    (*(volatile OS_U32* )0xBF8100C0u)
#define IPC0                    (*(volatile OS_U32* )0xBF810140u)
#define INTCON                  (*(volatile OS_U32* )0xBF810000u)
#define CTIE_BIT                0u
#define SYSTICK_PRIORITY        1u
#define SYSTICK_SUBPRIORITY     0u

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

/*********************************************************************
*
*       _InitVIC()
*
*  Function description
*    Set the CP0 registers for multi-vector interrupt Place
*    Vector Spacing to 32 bytes
*/
static void __attribute__((nomips16)) _InitVIC(void) {
  unsigned int temp;
  OS_MIPS_SetIntCtl(0x00000020u); // Set the Vector Spacing to non-zero value
  temp  = OS_MIPS_GetCause();
  temp |= 0x00800000u;            // Set IV
  OS_MIPS_SetCause(temp);         // Update Cause
  temp  = OS_MIPS_GetStatus();
  temp &= 0xFFBFFFFDu;            // Clear BEV and EXL
  OS_MIPS_SetStatus(temp);
  INTCON |= 0x1000u;              // Set MVEC bit
}

/*********************************************************************
*
*       _OS_GetHWTimerCycles()
*
*  Function description
*    Returns the current hardware timer count value
*
*  Return value
*    Current timer count value
*/
static unsigned int _OS_GetHWTimerCycles(void) {
  OS_I32 Cnt;
  Cnt  = (OS_MIPS_GetCompare() - OS_MIPS_GetCount());
  if (Cnt < 0) {  // Missed a counter interrupt, adjust time
    Cnt = 0 - Cnt;
  } else {
    Cnt = OS_TIMER_RELOAD - Cnt;
  }
  return (unsigned int)Cnt;
}

/*********************************************************************
*
*       _OS_GetHWTimer_IntPending()
*
*  Function description
*    Returns if the hardware timer interrupt pending flag is set
*
*  Return value
*    == 0; Interrupt pending flag not set
*    != 0: Interrupt pending flag set
*/
static unsigned int _OS_GetHWTimer_IntPending(void) {
  OS_I32 Cnt;
  Cnt  = OS_MIPS_GetCompare() - OS_MIPS_GetCount();
  if (Cnt < 0) {
    return 1;
  } else {
    return 0;
  }
}

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

void __attribute__( (interrupt(IPL1SOFT), vector(0))) OS_SysTick_ISR( void );

/*********************************************************************
*
*       OS_Systick()
*
*  Function description
*    This is the code that gets called when the processor receives a
*    _SysTick exception. SysTick is used as OS timer tick.
*/
void OS_Systick(void) {
  OS_I32 t, t1;
  IFS0 &= ~0x00000001;      // reset core timer interrupt pending flag
  OS_EnterNestableInterrupt();

  t  = OS_MIPS_GetCompare();
  do {
    t += OS_TIMER_RELOAD;
    OS_TICK_Handle();
    t1 = OS_MIPS_GetCount() + 1000;
  } while ((t - t1) < 0);
  OS_MIPS_SetCompare(t);

  OS_LeaveNestableInterrupt();
}

/*********************************************************************
*
*       OS_InitHW()
*
*  Function description
*    Initialize the hardware (timer) required for embOS to run.
*    May be modified, if an other timer should be used
*/
void OS_InitHW(void) {
  //
  // Structure with information about timer frequency, tick frequency, etc.
  // for micro second precise system time.
  //
  OS_SYSTIMER_CONFIG SysTimerConfig = {OS_PCLK_TIMER, OS_TICK_FREQ, 1, _OS_GetHWTimerCycles, _OS_GetHWTimer_IntPending};

  OS_IncDI();
  //
  // Set wait states and enable prefetch buffer
  //
  PRECON = 0u
         | (2u << 0u)  // 2 wait states
         | (3u << 4u); // Enable prefetch for instructions + data
  //
  // Initialize timer for embOS, assuming PLL is already initialized
  //
  OS_MIPS_SetCompare(OS_MIPS_GetCount() + OS_TIMER_RELOAD);        // Set compare register
  _InitVIC();                                                      // Setup vectored interrupt controller
  IPC0 |= (SYSTICK_PRIORITY << 2u) | SYSTICK_SUBPRIORITY;          // Set priority
  IEC0 |= (1u << CTIE_BIT);                                        // Enable core time interrupt in interrupt vector controller
  //
  // Setup values for usec precise system time functions
  //
  OS_Config_SysTimer(&SysTimerConfig);
  //
  // Initialize the optional communication for embOSView
  //
#if (OS_VIEW_IFSELECT != OS_VIEW_DISABLED)
  OS_COM_Init();
#endif
  OS_DecRI();
}

/*********************************************************************
*
*       OS_Idle()
*
*  Function description
*       This is basically the "core" of the idle loop.
*       This core loop can be changed, but:
*       The idle loop does not have a stack of its own, therefore no
*       functionality should be implemented that relies on the stack
*       to be preserved. However, a simple program loop can be programmed
*       (like toggling an output or incrementing a counter)
*/
void OS_Idle(void) {    // Idle loop: No task is ready to execute
  while (1) {           // Nothing to do ... wait for interrupt
    #if (OS_DEBUG == 0)
                        // Switch CPU into sleep mode
    #endif
  }
}

/*********************************************************************
*
*       OS_GetTime_Cycles()
*
*  Function description
*    This routine is required for task-info via embOSView or high
*    resolution time measurement functions.
*    It returns the system time in timer clock cycles.
*/
OS_U32 OS_GetTime_Cycles(void) {
  OS_U32 Time;
  OS_I32 Cnt;

  Time = OS_GetTime32();
  Cnt  = (OS_MIPS_GetCompare() - OS_MIPS_GetCount());
  if (Cnt < 0) {  // Missed a counter interrupt, adjust time
    Time++;
    Cnt = 0 - Cnt;
  } else {
    Cnt = OS_TIMER_RELOAD - Cnt;
  }
  return (OS_TIMER_RELOAD * Time) + Cnt;
}

/*********************************************************************
*
*       OS_ConvertCycles2us()
*
*  Function description
*    Convert Cycles into micro seconds.
*
*    If your clock frequency is not a multiple of 1 MHz,
*    you may have to modify this routine in order to get proper
*    diagnostics.
*
*    This routine is required for profiling or high resolution time
*    measurement only. It does not affect operation of the OS.
*/
OS_U32 OS_ConvertCycles2us(OS_U32 Cycles) {
  return (Cycles / (OS_TIMER_RELOAD / 1000u));
}

/*********************************************************************
*
*       Optional communication with embOSView
*
**********************************************************************
*/
#if (OS_VIEW_IFSELECT == OS_VIEW_DISABLED)

/*********************************************************************
*
*       OS_COM_Send1()
*
*  Function description
*    Dummy routine.
*/
void OS_COM_Send1(OS_U8 c) {
  OS_USEPARA(c);           // Avoid compiler warning
  OS_COM_ClearTxActive();  // Let embOS know that Tx is not busy
}

#else
  #error "Selected embOSView interface is currently not supported."
#endif

/****** End Of File *************************************************/
