/*********************************************************************
*                SEGGER Microcontroller GmbH & Co. KG                *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2017 SEGGER Microcontroller GmbH & Co. KG         *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 4.34                                             *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : BSP.c
Purpose : BSP for PIC32MZ2048EC/EF and PIC32MZ starter kit
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "BSP.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/
#define TRISH         (*(volatile unsigned int *) (0xBF860710u)) // Port H data direction register
#define PORTH         (*(volatile unsigned int *) (0xBF860720u)) // Port H data register
#define PORTHCLR      (*(volatile unsigned int *) (0xBF860724u)) // Port H data clear register
#define PORTHSET      (*(volatile unsigned int *) (0xBF860728u)) // Port H data set register
#define LATH          (*(volatile unsigned int *) (0xBF860730u)) // Port H latch register

#define NUM_LEDS      (3)
#define LED0_BIT      (0u)                                       // LED RH0
#define LED1_BIT      (1u)                                       // LED RH1
#define LED2_BIT      (2u)                                       // LED RH2
#define LEDMASK_ALL   ((1u << LED0_BIT) | (1u << LED1_BIT) | (1u << LED2_BIT))

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_Init()
*/
void BSP_Init(void) {
  TRISH   &= ~LEDMASK_ALL; // Enable LED output ports
  PORTHCLR = LEDMASK_ALL;  // Switch off all LEDs
}

/*********************************************************************
*
*       BSP_SetLED()
*/
void BSP_SetLED(int Index) {
  if (Index < NUM_LEDS) {
    PORTHSET = (1u << Index);
  }
}

/*********************************************************************
*
*       BSP_ClrLED()
*/
void BSP_ClrLED(int Index) {
  if (Index < NUM_LEDS) {
    PORTHCLR = (1u << Index);
  }
}

/*********************************************************************
*
*       BSP_ToggleLED()
*/
void BSP_ToggleLED(int Index) {
  if (Index < NUM_LEDS) {
    if (LATH & (1u << Index)) {
      PORTH = LATH & ~(1u << Index);
    } else {
      PORTH = LATH | (1u << Index);
    }
  }
}

/****** End Of File *************************************************/
